@@ -24,6 +24,11 @@
  */
 
 #include <errno.h>
+#include <fcntl.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #include <gtk/gtk.h>
 
@@ -35,7 +40,7 @@
 
 #ifdef USE_DBUS
 #include "audctrl.h"
-#include "dbus-service.h"
+#include "dbus.h"
 #endif
 
 #ifdef USE_EGGSM
@@ -57,20 +62,20 @@
 
 #define AUTOSAVE_INTERVAL 300 /* seconds */
 
-gboolean headless;
+bool_t headless;
 
 static struct {
-    gchar **filenames;
-    gint session;
-    gboolean play, stop, pause, fwd, rew, play_pause, show_jump_box;
-    gboolean enqueue, mainwin, remote;
-    gboolean enqueue_to_temp;
-    gboolean version;
-    gboolean verbose;
-    gchar *previous_session_id;
+    char **filenames;
+    int session;
+    bool_t play, stop, pause, fwd, rew, play_pause, show_jump_box;
+    bool_t enqueue, mainwin, remote;
+    bool_t enqueue_to_temp;
+    bool_t version;
+    bool_t verbose;
+    char *previous_session_id;
 } options;
 
-static gchar * aud_paths[AUD_PATH_COUNT];
+static char * aud_paths[AUD_PATH_COUNT];
 
 static void make_dirs(void)
 {
@@ -84,12 +89,12 @@ static void make_dirs(void)
     make_directory(aud_paths[AUD_PATH_PLAYLISTS_DIR], mode755);
 }
 
-static void normalize_path (gchar * path)
+static void normalize_path (char * path)
 {
 #ifdef _WIN32
     string_replace_char (path, '/', '\\');
 #endif
-    gint len = strlen (path);
+    int len = strlen (path);
 #ifdef _WIN32
     if (len > 3 && path[len - 1] == '\\') /* leave "C:\" */
 #else
@@ -98,13 +103,13 @@ static void normalize_path (gchar * path)
         path[len - 1] = 0;
 }
 
-static gchar * last_path_element (gchar * path)
+static char * last_path_element (char * path)
 {
-    gchar * slash = strrchr (path, G_DIR_SEPARATOR);
+    char * slash = strrchr (path, G_DIR_SEPARATOR);
     return (slash && slash[1]) ? slash + 1 : NULL;
 }
 
-static void strip_path_element (gchar * path, gchar * elem)
+static void strip_path_element (char * path, char * elem)
 {
 #ifdef _WIN32
     if (elem > path + 3)
@@ -116,11 +121,11 @@ static void strip_path_element (gchar * path, gchar * elem)
         elem[0] = 0; /* leave [drive letter and] leading slash */
 }
 
-static void relocate_path (gchar * * pathp, const gchar * old, const gchar * new)
+static void relocate_path (char * * pathp, const char * old, const char * new)
 {
-    gchar * path = * pathp;
-    gint oldlen = strlen (old);
-    gint newlen = strlen (new);
+    char * path = * pathp;
+    int oldlen = strlen (old);
+    int newlen = strlen (new);
 
     if (oldlen && old[oldlen - 1] == G_DIR_SEPARATOR)
         oldlen --;
@@ -160,8 +165,8 @@ static void relocate_paths (void)
 
     /* Compare the compile-time path to the executable and the actual path to
      * see if we have been moved. */
-    gchar * old = g_strdup (aud_paths[AUD_PATH_BIN_DIR]);
-    gchar * new = get_path_to_self ();
+    char * old = g_strdup (aud_paths[AUD_PATH_BIN_DIR]);
+    char * new = get_path_to_self ();
     if (! new)
     {
 ERR:
@@ -172,14 +177,14 @@ ERR:
     normalize_path (new);
 
     /* Strip the name of the executable file, leaving the path. */
-    gchar * base = last_path_element (new);
+    char * base = last_path_element (new);
     if (! base)
         goto ERR;
     strip_path_element (new, base);
 
     /* Strip innermost folder names from both paths as long as they match.  This
      * leaves a compile-time prefix and a run-time one to replace it with. */
-    gchar * a, * b;
+    char * a, * b;
     while ((a = last_path_element (old)) && (b = last_path_element (new)) &&
 #ifdef _WIN32
      ! strcasecmp (a, b))
@@ -207,8 +212,8 @@ static void init_paths (void)
 {
     relocate_paths ();
 
-    const gchar * xdg_config_home = g_get_user_config_dir ();
-    const gchar * xdg_data_home = g_get_user_data_dir ();
+    const char * xdg_config_home = g_get_user_config_dir ();
+    const char * xdg_data_home = g_get_user_data_dir ();
 
 #ifdef _WIN32
     /* Some libraries (libmcs) and plugins (filewriter) use these variables,
@@ -222,14 +227,13 @@ static void init_paths (void)
     aud_paths[AUD_PATH_USER_DIR] = g_build_filename(xdg_config_home, "audacious", NULL);
     aud_paths[AUD_PATH_USER_PLUGIN_DIR] = g_build_filename(xdg_data_home, "audacious", "Plugins", NULL);
     aud_paths[AUD_PATH_PLAYLISTS_DIR] = g_build_filename(aud_paths[AUD_PATH_USER_DIR], "playlists", NULL);
-    aud_paths[AUD_PATH_PLAYLIST_FILE] = g_build_filename(aud_paths[AUD_PATH_USER_DIR], "playlist.xspf", NULL);
     aud_paths[AUD_PATH_GTKRC_FILE] = g_build_filename(aud_paths[AUD_PATH_USER_DIR], "gtkrc", NULL);
 
-    for (gint i = 0; i < AUD_PATH_COUNT; i ++)
+    for (int i = 0; i < AUD_PATH_COUNT; i ++)
         AUDDBG ("Data path: %s\n", aud_paths[i]);
 }
 
-const gchar * get_path (gint id)
+const char * get_path (int id)
 {
     g_return_val_if_fail (id >= 0 && id < AUD_PATH_COUNT, NULL);
     return aud_paths[id];
@@ -253,7 +257,7 @@ static GOptionEntry cmd_entries[] = {
     {NULL},
 };
 
-static void parse_options (gint * argc, gchar *** argv)
+static void parse_options (int * argc, char *** argv)
 {
     GOptionContext *context;
     GError *error = NULL;
@@ -281,9 +285,9 @@ static void parse_options (gint * argc, gchar *** argv)
     verbose = options.verbose;
 }
 
-static gboolean get_lock (void)
+static bool_t get_lock (void)
 {
-    gchar * path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "lock", aud_paths[AUD_PATH_USER_DIR]);
+    char * path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "lock", aud_paths[AUD_PATH_USER_DIR]);
     int handle = open (path, O_RDWR | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
 
     if (handle < 0)
@@ -302,40 +306,47 @@ static gboolean get_lock (void)
 
 static void release_lock (void)
 {
-    gchar * path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "lock", aud_paths[AUD_PATH_USER_DIR]);
+    char * path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "lock", aud_paths[AUD_PATH_USER_DIR]);
     unlink (path);
     g_free (path);
 }
 
-static GList * convert_filenames (void)
+static Index * convert_filenames (void)
 {
     if (! options.filenames)
         return NULL;
 
-    gchar * * f = options.filenames;
-    GList * list = NULL;
-    gchar * cur = g_get_current_dir ();
+    Index * filenames = index_new ();
+    char * * f = options.filenames;
+    char * cur = g_get_current_dir ();
 
-    for (gint i = 0; f[i]; i ++)
+    for (int i = 0; f[i]; i ++)
     {
-        gchar * uri;
+        char * uri = NULL;
 
         if (strstr (f[i], "://"))
-            uri = g_strdup (f[i]);
+            uri = str_get (f[i]);
         else if (g_path_is_absolute (f[i]))
-            uri = filename_to_uri (f[i]);
+        {
+            char * tmp = filename_to_uri (f[i]);
+            uri = str_get (tmp);
+            free (tmp);
+        }
         else
         {
-            gchar * tmp = g_build_filename (cur, f[i], NULL);
-            uri = filename_to_uri (tmp);
-            g_free (tmp);
+            char * tmp = g_build_filename (cur, f[i], NULL);
+            char * tmp2 = filename_to_uri (tmp);
+            uri = str_get (tmp2);
+            free (tmp);
+            free (tmp2);
         }
 
-        list = g_list_prepend (list, uri);
+        if (uri)
+            index_append (filenames, uri);
     }
 
     g_free (cur);
-    return g_list_reverse (list);
+    return filenames;
 }
 
 static void do_remote (void)
@@ -345,16 +356,21 @@ static void do_remote (void)
 
     if (session && audacious_remote_is_running (session))
     {
-        GList * list = convert_filenames ();
+        Index * filenames = convert_filenames ();
 
         /* if no command line options, then present running instance */
-        if (! (list || options.play || options.pause || options.play_pause ||
+        if (! (filenames || options.play || options.pause || options.play_pause ||
          options.stop || options.rew || options.fwd || options.show_jump_box ||
          options.mainwin))
             options.mainwin = TRUE;
 
-        if (list)
+        if (filenames)
         {
+            GList * list = NULL;
+
+            for (int f = index_count (filenames); f --; )
+                list = g_list_prepend (list, index_get (filenames, f));
+
             if (options.enqueue_to_temp)
                 audacious_remote_playlist_open_list_to_temp (session, list);
             else if (options.enqueue)
@@ -362,8 +378,12 @@ static void do_remote (void)
             else
                 audacious_remote_playlist_open_list (session, list);
 
-            g_list_foreach (list, (GFunc) g_free, NULL);
             g_list_free (list);
+
+            for (int f = 0; f < index_count (filenames); f ++)
+                str_unref (index_get (filenames, f));
+
+            index_free (filenames);
         }
 
         if (options.play)
@@ -392,26 +412,23 @@ static void do_remote (void)
 
 static void do_commands (void)
 {
-    gboolean resume = get_bool (NULL, "resume_playback_on_startup");
+    bool_t resume = get_bool (NULL, "resume_playback_on_startup");
 
-    GList * list = convert_filenames ();
-    if (list)
+    Index * filenames = convert_filenames ();
+    if (filenames)
     {
         if (options.enqueue_to_temp)
         {
-            drct_pl_open_temp_list (list);
+            drct_pl_open_temp_list (filenames);
             resume = FALSE;
         }
         else if (options.enqueue)
-            drct_pl_add_list (list, -1);
+            drct_pl_add_list (filenames, -1);
         else
         {
-            drct_pl_open_list (list);
+            drct_pl_open_list (filenames);
             resume = FALSE;
         }
-
-        g_list_foreach (list, (GFunc) g_free, NULL);
-        g_list_free (list);
     }
 
     if (resume)
@@ -442,15 +459,13 @@ static void init_one (void)
     bind_textdomain_codeset (PACKAGE "-plugins", "UTF-8");
     textdomain (PACKAGE);
 
-    mowgli_init ();
-
 #ifdef USE_EGGSM
     egg_sm_client_set_mode (EGG_SM_CLIENT_MODE_NORMAL);
     egg_set_desktop_file (aud_paths[AUD_PATH_DESKTOP_FILE]);
 #endif
 }
 
-static void init_two (gint * p_argc, gchar * * * p_argv)
+static void init_two (int * p_argc, char * * * p_argv)
 {
     if (! headless)
     {
@@ -462,9 +477,6 @@ static void init_two (gint * p_argc, gchar * * * p_argv)
         gtk_init (p_argc, p_argv);
     }
 
-    hook_init ();
-    tag_init ();
-
     config_load ();
     chardet_init ();
 
@@ -472,7 +484,6 @@ static void init_two (gint * p_argc, gchar * * * p_argv)
     vfs_set_verbose (verbose);
 
     eq_init ();
-    register_interface_hooks ();
 
 #ifdef HAVE_SIGWAIT
     signals_init ();
@@ -486,6 +497,7 @@ static void init_two (gint * p_argc, gchar * * * p_argv)
 
     playlist_init ();
     adder_init ();
+    art_init ();
     load_playlists ();
 
 #ifdef USE_DBUS
@@ -514,12 +526,18 @@ static void shut_down (void)
     AUDDBG ("Stopping playback.\n");
     if (playback_get_playing ())
     {
-        gboolean stop_after_song = get_bool (NULL, "stop_after_current_song");
+        bool_t stop_after_song = get_bool (NULL, "stop_after_current_song");
         playback_stop ();
         set_bool (NULL, "stop_after_current_song", stop_after_song);
     }
 
+#ifdef USE_DBUS
+    cleanup_dbus ();
+#endif
+
     adder_cleanup ();
+    art_cleanup ();
+    history_cleanup ();
     playlist_end ();
 
     AUDDBG ("Unloading lowlevel plugins.\n");
@@ -529,10 +547,14 @@ static void shut_down (void)
     config_save ();
     config_cleanup ();
 
+    eq_cleanup ();
+
+    strpool_shutdown ();
+
     gdk_threads_leave ();
 }
 
-gboolean do_autosave (void)
+bool_t do_autosave (void)
 {
     AUDDBG ("Saving configuration.\n");
     hook_call ("config save", NULL);
@@ -541,7 +563,7 @@ gboolean do_autosave (void)
     return TRUE;
 }
 
-gint main(gint argc, gchar ** argv)
+int main(int argc, char ** argv)
 {
     init_one ();
     parse_options (& argc, & argv);
@@ -560,9 +582,10 @@ gint main(gint argc, gchar ** argv)
 
     AUDDBG ("Startup complete.\n");
     g_timeout_add_seconds (AUTOSAVE_INTERVAL, (GSourceFunc) do_autosave, NULL);
-    hook_associate ("quit", (HookFunction) gtk_main_quit, NULL);
 
+    hook_associate ("quit", (HookFunction) gtk_main_quit, NULL);
     gtk_main ();
+    hook_dissociate ("quit", (HookFunction) gtk_main_quit);
 
     shut_down ();
     release_lock ();
