@@ -19,8 +19,10 @@
  * using our public API to be a derived work.
  */
 
+#include <glib.h>
 #include <math.h>
-#include <libaudcore/audio.h>
+#include <pthread.h>
+#include <string.h>
 
 #include "debug.h"
 #include "effect.h"
@@ -35,7 +37,7 @@
 
 static OutputPlugin * cop = NULL;
 
-void output_get_volume (gint * l, gint * r)
+void output_get_volume (int * l, int * r)
 {
     if (get_bool (NULL, "software_volume_control"))
     {
@@ -51,7 +53,7 @@ void output_get_volume (gint * l, gint * r)
     }
 }
 
-void output_set_volume (gint l, gint r)
+void output_set_volume (int l, int r)
 {
     if (get_bool (NULL, "software_volume_control"))
     {
@@ -62,11 +64,11 @@ void output_set_volume (gint l, gint r)
         cop->set_volume (l, r);
 }
 
-static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
-static gboolean locked = FALSE;
+static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+static bool_t locked = FALSE;
 
-#define LOCK do {g_static_mutex_lock (& mutex); locked = TRUE;} while (0)
-#define UNLOCK do {locked = FALSE; g_static_mutex_unlock (& mutex);} while (0)
+#define LOCK do {pthread_mutex_lock (& mutex); locked = TRUE;} while (0)
+#define UNLOCK do {locked = FALSE; pthread_mutex_unlock (& mutex);} while (0)
 #define LOCKED g_return_if_fail (locked)
 #define LOCKED_RET(a) g_return_val_if_fail (locked, a)
 #define LOCK_VIS do {vis_runner_lock (); LOCK;} while (0)
@@ -74,14 +76,14 @@ static gboolean locked = FALSE;
 #define LOCKED_VIS g_return_if_fail (locked && vis_runner_locked ())
 #define LOCKED_VIS_RET(a) g_return_val_if_fail (locked && vis_runner_locked (), a)
 
-static gboolean opened = FALSE;
-static gboolean leave_open = FALSE;
+static bool_t opened = FALSE;
+static bool_t leave_open = FALSE;
 
-static gboolean waiting, aborted, paused;
-static gint decoder_format, decoder_channels, decoder_rate, effect_channels,
+static bool_t waiting, aborted, paused;
+static int decoder_format, decoder_channels, decoder_rate, effect_channels,
  effect_rate, output_format, output_channels, output_rate;
-static gint64 frames_written;
-static gboolean have_replay_gain;
+static int64_t frames_written;
+static bool_t have_replay_gain;
 static ReplayGainInfo replay_gain_info;
 
 static void reset_time (void)
@@ -108,7 +110,7 @@ static void real_close (void)
     leave_open = FALSE;
 }
 
-static gboolean open_audio (gint format, gint rate, gint channels)
+static bool_t open_audio (int format, int rate, int channels)
 {
     LOCKED_VIS_RET (FALSE);
     g_return_val_if_fail (! opened, FALSE);
@@ -135,7 +137,7 @@ static gboolean open_audio (gint format, gint rate, gint channels)
             real_close ();
         }
 
-        gint depth = get_int (NULL, "output_bit_depth");
+        int depth = get_int (NULL, "output_bit_depth");
         output_format = (depth == 32) ? FMT_S32_NE : (depth == 24) ? FMT_S24_NE
          : (depth == 16) ? FMT_S16_NE : FMT_FLOAT;
         output_channels = effect_channels;
@@ -158,11 +160,11 @@ static gboolean open_audio (gint format, gint rate, gint channels)
     return opened;
 }
 
-static gboolean output_open_audio (gint format, gint rate, gint channels)
+static bool_t output_open_audio (int format, int rate, int channels)
 {
     g_return_val_if_fail (cop != NULL, FALSE);
     LOCK_VIS;
-    gboolean success = open_audio (format, rate, channels);
+    bool_t success = open_audio (format, rate, channels);
     UNLOCK_VIS;
     return success;
 }
@@ -190,12 +192,12 @@ static void output_set_replaygain_info (ReplayGainInfo * info)
     UNLOCK;
 }
 
-static void apply_replay_gain (gfloat * data, gint samples)
+static void apply_replay_gain (float * data, int samples)
 {
     if (! get_bool (NULL, "enable_replay_gain"))
         return;
 
-    gfloat factor = powf (10, get_double (NULL, "replay_gain_preamp") / 20);
+    float factor = powf (10, get_double (NULL, "replay_gain_preamp") / 20);
 
     if (have_replay_gain)
     {
@@ -223,22 +225,22 @@ static void apply_replay_gain (gfloat * data, gint samples)
         audio_amplify (data, 1, samples, & factor);
 }
 
-static void apply_software_volume (gfloat * data, gint channels, gint frames)
+static void apply_software_volume (float * data, int channels, int frames)
 {
-    gfloat left_factor, right_factor;
-    gfloat factors[channels];
-    gint channel;
+    float left_factor, right_factor;
+    float factors[channels];
+    int channel;
 
     if (! get_bool (NULL, "software_volume_control"))
         return;
 
-    gint l = get_int (NULL, "sw_volume_left");
-    gint r = get_int (NULL, "sw_volume_right");
+    int l = get_int (NULL, "sw_volume_left");
+    int r = get_int (NULL, "sw_volume_right");
     if (l == 100 && r == 100)
         return;
 
-    left_factor = (l == 0) ? 0 : powf (10, (gfloat) SW_VOLUME_RANGE * (l - 100) / 100 / 20);
-    right_factor = (r == 0) ? 0 : powf (10, (gfloat) SW_VOLUME_RANGE * (r - 100) / 100 / 20);
+    left_factor = (l == 0) ? 0 : powf (10, (float) SW_VOLUME_RANGE * (l - 100) / 100 / 20);
+    right_factor = (r == 0) ? 0 : powf (10, (float) SW_VOLUME_RANGE * (r - 100) / 100 / 20);
 
     if (channels == 2)
     {
@@ -254,7 +256,7 @@ static void apply_software_volume (gfloat * data, gint channels, gint frames)
     audio_amplify (data, channels, frames, factors);
 }
 
-static void write_processed (void * data, gint samples)
+static void write_processed (void * data, int samples)
 {
     LOCKED_VIS;
 
@@ -279,7 +281,7 @@ static void write_processed (void * data, gint samples)
 
     while (! aborted)
     {
-        gint ready = (cop->buffer_free != NULL) ? cop->buffer_free () /
+        int ready = (cop->buffer_free != NULL) ? cop->buffer_free () /
          FMT_SIZEOF (output_format) : output_channels * (output_rate / 50);
         ready = MIN (ready, samples);
         cop->write_audio (data, FMT_SIZEOF (output_format) * ready);
@@ -304,19 +306,19 @@ static void write_processed (void * data, gint samples)
     g_free (allocated);
 }
 
-static void write_audio (void * data, gint size)
+static void write_audio (void * data, int size)
 {
     LOCKED;
     g_return_if_fail (opened && ! waiting);
 
-    gint samples = size / FMT_SIZEOF (decoder_format);
+    int samples = size / FMT_SIZEOF (decoder_format);
     frames_written += samples / decoder_channels;
 
     void * allocated = NULL;
 
     if (decoder_format != FMT_FLOAT)
     {
-        gfloat * new = g_malloc (sizeof (gfloat) * samples);
+        float * new = g_malloc (sizeof (float) * samples);
         audio_from_int (data, decoder_format, new, samples);
         data = new;
         g_free (allocated);
@@ -324,7 +326,7 @@ static void write_audio (void * data, gint size)
     }
 
     apply_replay_gain (data, samples);
-    gfloat * fdata = data;
+    float * fdata = data;
     effect_process (& fdata, & samples);
     data = fdata;
 
@@ -338,7 +340,7 @@ static void write_audio (void * data, gint size)
     g_free (allocated);
 }
 
-static void output_write_audio (void * data, gint size)
+static void output_write_audio (void * data, int size)
 {
     g_return_if_fail (cop != NULL);
     LOCK_VIS;
@@ -367,7 +369,7 @@ static void output_close_audio (void)
     UNLOCK_VIS;
 }
 
-static void do_pause (gboolean p)
+static void do_pause (bool_t p)
 {
     LOCKED_VIS;
     g_return_if_fail (opened);
@@ -376,7 +378,7 @@ static void do_pause (gboolean p)
     paused = p;
 }
 
-static void output_pause (gboolean p)
+static void output_pause (bool_t p)
 {
     g_return_if_fail (cop != NULL);
     LOCK_VIS;
@@ -384,7 +386,7 @@ static void output_pause (gboolean p)
     UNLOCK_VIS;
 }
 
-static void flush (gint time)
+static void flush (int time)
 {
     LOCKED_VIS;
     g_return_if_fail (opened);
@@ -408,10 +410,10 @@ static void flush (gint time)
         cop->flush (effect_decoder_to_output_time (time));
     }
 
-    frames_written = time * (gint64) decoder_rate / 1000;
+    frames_written = time * (int64_t) decoder_rate / 1000;
 }
 
-static void output_flush (gint time)
+static void output_flush (int time)
 {
     g_return_if_fail (cop != NULL);
     LOCK_VIS;
@@ -419,18 +421,18 @@ static void output_flush (gint time)
     UNLOCK_VIS;
 }
 
-static gint written_time (void)
+static int written_time (void)
 {
     LOCKED_RET (0);
     g_return_val_if_fail (opened && ! waiting, 0);
-    return frames_written * (gint64) 1000 / decoder_rate;
+    return frames_written * (int64_t) 1000 / decoder_rate;
 }
 
-static gint output_written_time (void)
+static int output_written_time (void)
 {
     g_return_val_if_fail (cop != NULL, 0);
     LOCK;
-    gint time = written_time ();
+    int time = written_time ();
     UNLOCK;
     return time;
 }
@@ -438,8 +440,8 @@ static gint output_written_time (void)
 static void write_buffers (void)
 {
     LOCKED;
-    gfloat * data = NULL;
-    gint samples = 0;
+    float * data = NULL;
+    int samples = 0;
     effect_finish (& data, & samples);
     write_processed (data, samples);
 }
@@ -456,7 +458,7 @@ static void set_leave_open (void)
     }
 }
 
-static gboolean output_buffer_playing (void)
+static bool_t output_buffer_playing (void)
 {
     g_return_val_if_fail (cop != NULL, FALSE);
     LOCK_VIS;
@@ -495,19 +497,19 @@ const struct OutputAPI output_api =
     .abort_write = output_abort_write,
 };
 
-static gint output_time (void)
+static int output_time (void)
 {
     LOCKED_RET (0);
     g_return_val_if_fail (opened || leave_open, 0);
     return cop->output_time ();
 }
 
-gint get_output_time (void)
+int get_output_time (void)
 {
     g_return_val_if_fail (cop != NULL, 0);
     LOCK;
 
-    gint time = 0;
+    int time = 0;
     if (opened)
     {
         time = effect_output_to_decoder_time (output_time ());
@@ -518,11 +520,11 @@ gint get_output_time (void)
     return time;
 }
 
-gint get_raw_output_time (void)
+int get_raw_output_time (void)
 {
     g_return_val_if_fail (cop != NULL, 0);
     LOCK;
-    gint time = output_time ();
+    int time = output_time ();
     UNLOCK;
     return time;
 }
@@ -542,7 +544,7 @@ void output_drain (void)
     UNLOCK_VIS;
 }
 
-static gboolean probe_cb (PluginHandle * p, PluginHandle * * pp)
+static bool_t probe_cb (PluginHandle * p, PluginHandle * * pp)
 {
     OutputPlugin * op = plugin_get_header (p);
     g_return_val_if_fail (op != NULL && op->init != NULL, TRUE);
@@ -569,7 +571,7 @@ PluginHandle * output_plugin_get_current (void)
     return (cop != NULL) ? plugin_by_header (cop) : NULL;
 }
 
-gboolean output_plugin_set_current (PluginHandle * plugin)
+bool_t output_plugin_set_current (PluginHandle * plugin)
 {
     if (cop != NULL)
     {
