@@ -24,6 +24,7 @@
  */
 
 #include <dirent.h>
+#include <sys/stat.h>
 #include <unistd.h>
 
 #ifdef _WIN32
@@ -42,7 +43,6 @@
 #include <errno.h>
 
 #include <libaudcore/audstrings.h>
-#include <libaudcore/stringpool.h>
 
 #include "debug.h"
 #include "i18n.h"
@@ -50,7 +50,7 @@
 #include "plugins.h"
 #include "util.h"
 
-gboolean dir_foreach (const gchar * path, DirForeachFunc func, void * user)
+bool_t dir_foreach (const char * path, DirForeachFunc func, void * user)
 {
     DIR * dir = opendir (path);
     if (! dir)
@@ -62,8 +62,8 @@ gboolean dir_foreach (const gchar * path, DirForeachFunc func, void * user)
         if (entry->d_name[0] == '.')
             continue;
 
-        gchar * full = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "%s", path, entry->d_name);
-        gboolean stop = func (full, entry->d_name, user);
+        char * full = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "%s", path, entry->d_name);
+        bool_t stop = func (full, entry->d_name, user);
         g_free (full);
 
         if (stop)
@@ -74,37 +74,34 @@ gboolean dir_foreach (const gchar * path, DirForeachFunc func, void * user)
     return TRUE;
 }
 
-gchar * construct_uri (const gchar * string, const gchar * playlist_name)
+char * construct_uri (const char * string, const char * playlist_name)
 {
-    gchar *filename = g_strdup(string);
-    gchar *uri = NULL;
+    /* URI */
+    if (strstr (string, "://"))
+        return strdup (string);
 
-    /* try to translate dos path */
-    convert_dos_path(filename); /* in place replacement */
+    /* absolute filename (assumed UTF-8) */
+#ifdef _WIN32
+    if (string[0] && string[1] == ':' && string[2] == '\\')
+#else
+    if (string[0] == '/')
+#endif
+        return filename_to_uri (string);
 
-    // make full path uri here
-    // case 1: filename is raw full path or uri
-    if (filename[0] == '/' || strstr(filename, "://")) {
-        uri = g_filename_to_uri(filename, NULL, NULL);
-        if(!uri)
-            uri = g_strdup(filename);
-    }
-    // case 2: filename is not raw full path nor uri
-    // make full path by replacing last part of playlist path with filename.
-    else
-    {
-        const gchar * slash = strrchr (playlist_name, '/');
-        if (slash)
-            uri = g_strdup_printf ("%.*s/%s", (gint) (slash - playlist_name),
-             playlist_name, filename);
-    }
+    /* relative filename (assumed UTF-8) */
+    const char * slash = strrchr (playlist_name, '/');
+    if (! slash)
+        return NULL;
 
-    g_free (filename);
-    return uri;
+    int pathlen = slash + 1 - playlist_name;
+    char buf[pathlen + 3 * strlen (string) + 1];
+    memcpy (buf, playlist_name, pathlen);
+    str_encode_percent (string, -1, buf + pathlen);
+    return strdup (buf);
 }
 
 /* local files -- not URI's */
-gint file_get_mtime (const gchar * filename)
+int file_get_mtime (const char * filename)
 {
     struct stat info;
 
@@ -115,7 +112,7 @@ gint file_get_mtime (const gchar * filename)
 }
 
 void
-make_directory(const gchar * path, mode_t mode)
+make_directory(const char * path, mode_t mode)
 {
     if (g_mkdir_with_parents(path, mode) == 0)
         return;
@@ -124,15 +121,52 @@ make_directory(const gchar * path, mode_t mode)
                g_strerror(errno));
 }
 
-gchar * get_path_to_self (void)
+char * write_temp_file (void * data, int64_t len)
+{
+    char * name = g_strdup_printf ("%s/audacious-temp-XXXXXX", g_get_tmp_dir ());
+
+    int handle = g_mkstemp (name);
+    if (handle < 0)
+    {
+        fprintf (stderr, "Error creating temporary file: %s\n", strerror (errno));
+        g_free (name);
+        return NULL;
+    }
+
+    while (len)
+    {
+        int64_t written = write (handle, data, len);
+        if (written < 0)
+        {
+            fprintf (stderr, "Error writing %s: %s\n", name, strerror (errno));
+            close (handle);
+            g_free (name);
+            return NULL;
+        }
+
+        data += written;
+        len -= written;
+    }
+
+    if (close (handle) < 0)
+    {
+        fprintf (stderr, "Error closing %s: %s\n", name, strerror (errno));
+        g_free (name);
+        return NULL;
+    }
+
+    return name;
+}
+
+char * get_path_to_self (void)
 {
 #if defined _WIN32 || defined HAVE_PROC_SELF_EXE
-    gint size = 256;
-    gchar * buf = g_malloc (size);
+    int size = 256;
+    char * buf = g_malloc (size);
 
     while (1)
     {
-        gint len;
+        int len;
 
 #ifdef _WIN32
         if (! (len = GetModuleFileName (NULL, buf, size)))
@@ -164,40 +198,40 @@ gchar * get_path_to_self (void)
 #endif
 }
 
-/* Strips various common top-level folders from a file name (not URI).  The
- * string passed will not be modified, but the string returned will share the
- * same memory.  Examples:
- *     "/home/john/folder/file.mp3"    -> "folder/file.mp3"
- *     "/folder/file.mp3"              -> "folder/file.mp3"
- *     "C:\Users\John\folder\file.mp3" -> "folder\file.mp3"
- *     "E:\folder\file.mp3"            -> "folder\file.mp3" */
+/* Strips various common top-level folders from a URI.  The string passed will
+ * not be modified, but the string returned will share the same memory.
+ * Examples:
+ *     "file:///home/john/folder/file.mp3" -> "folder/file.mp3"
+ *     "file:///folder/file.mp3"           -> "folder/file.mp3" */
 
-static gchar * skip_top_folders (gchar * name)
+static char * skip_top_folders (char * name)
 {
-    const gchar * home = getenv ("HOME");
-    if (! home)
-        goto NO_HOME;
+    static char * home;
+    static int len;
 
-    gint len = strlen (home);
-    if (len > 0 && home[len - 1] == G_DIR_SEPARATOR)
-        len --;
+    if (! home)
+    {
+        home = filename_to_uri (g_get_home_dir ());
+        len = strlen (home);
+
+        if (len > 0 && home[len - 1] == '/')
+            len --;
+    }
 
 #ifdef _WIN32
-    if (! strncasecmp (name, home, len) && name[len] == '\\')
+    if (! g_ascii_strncasecmp (name, home, len) && name[len] == '/')
 #else
     if (! strncmp (name, home, len) && name[len] == '/')
 #endif
         return name + len + 1;
 
-NO_HOME:
-#ifdef _WIN32
-    return (name[0] && name[1] == ':' && name[2] == '\\') ? name + 3 : name;
-#else
-    return (name[0] == '/') ? name + 1 : name;
-#endif
+    if (! strncmp (name, "file:///", 8))
+        return name + 8;
+
+    return name;
 }
 
-/* Divides a file name (not URI) into the base name, the lowest folder, and the
+/* Divides a URI into the base name, the lowest folder, and the
  * second lowest folder.  The string passed will be modified, and the strings
  * returned will use the same memory.  May return NULL for <first> and <second>.
  * Examples:
@@ -205,14 +239,14 @@ NO_HOME:
  *     "d/e.mp3"       -> "e", "d",  NULL
  *     "e.mp3"         -> "e", NULL, NULL */
 
-static void split_filename (gchar * name, gchar * * base, gchar * * first,
- gchar * * second)
+static void split_filename (char * name, char * * base, char * * first,
+ char * * second)
 {
     * first = * second = NULL;
 
-    gchar * c;
+    char * c;
 
-    if ((c = strrchr (name, G_DIR_SEPARATOR)))
+    if ((c = strrchr (name, '/')))
     {
         * base = c + 1;
         * c = 0;
@@ -223,7 +257,7 @@ static void split_filename (gchar * name, gchar * * base, gchar * * first,
         goto DONE;
     }
 
-    if ((c = strrchr (name, G_DIR_SEPARATOR)))
+    if ((c = strrchr (name, '/')))
     {
         * first = c + 1;
         * c = 0;
@@ -234,7 +268,7 @@ static void split_filename (gchar * name, gchar * * base, gchar * * first,
         goto DONE;
     }
 
-    if ((c = strrchr (name, G_DIR_SEPARATOR)))
+    if ((c = strrchr (name, '/')))
         * second = c + 1;
     else
         * second = name;
@@ -250,7 +284,7 @@ DONE:
  *     "http://some.domain.org/folder/file.mp3" -> "some.domain.org"
  *     "http://some.stream.fm:8000"             -> "some.stream.fm" */
 
-static gchar * stream_name (gchar * name)
+static char * stream_name (char * name)
 {
     if (! strncmp (name, "http://", 7))
         name += 7;
@@ -261,7 +295,7 @@ static gchar * stream_name (gchar * name)
     else
         return NULL;
 
-    gchar * c;
+    char * c;
 
     if ((c = strchr (name, '/')))
         * c = 0;
@@ -273,47 +307,66 @@ static gchar * stream_name (gchar * name)
     return name;
 }
 
+static char * get_nonblank_field (const Tuple * tuple, int field)
+{
+    char * str = tuple ? tuple_get_str (tuple, field, NULL) : NULL;
+
+    if (str && ! str[0])
+    {
+        str_unref (str);
+        str = NULL;
+    }
+
+    return str;
+}
+
+static char * str_get_decoded (char * str)
+{
+    if (! str)
+        return NULL;
+
+    str_decode_percent (str, -1, str);
+    return str_get (str);
+}
+
 /* Derives best guesses of title, artist, and album from a file name (URI) and
  * tuple (which may be NULL).  The returned strings are stringpooled or NULL. */
 
-void describe_song (const gchar * name, const Tuple * tuple, gchar * * _title,
- gchar * * _artist, gchar * * _album)
+void describe_song (const char * name, const Tuple * tuple, char * * _title,
+ char * * _artist, char * * _album)
 {
     /* Common folder names to skip */
-    static const gchar * const skip[] = {"music"};
+    static const char * const skip[] = {"music"};
 
-    const gchar * title = tuple ? tuple_get_string (tuple, FIELD_TITLE, NULL) : NULL;
-    const gchar * artist = tuple ? tuple_get_string (tuple, FIELD_ARTIST, NULL) : NULL;
-    const gchar * album = tuple ? tuple_get_string (tuple, FIELD_ALBUM, NULL) : NULL;
-
-    if (title && ! title[0])
-        title = NULL;
-    if (artist && ! artist[0])
-        artist = NULL;
-    if (album && ! album[0])
-        album = NULL;
-
-    gchar * copy = NULL;
+    char * title = get_nonblank_field (tuple, FIELD_TITLE);
+    char * artist = get_nonblank_field (tuple, FIELD_ARTIST);
+    char * album = get_nonblank_field (tuple, FIELD_ALBUM);
 
     if (title && artist && album)
-        goto DONE;
-
-    copy = uri_to_display (name);
-
-    if (! strncmp (name, "file://", 7))
     {
-        gchar * base, * first, * second;
-        split_filename (skip_top_folders (copy), & base, & first,
-         & second);
+DONE:
+        * _title = title;
+        * _artist = artist;
+        * _album = album;
+        return;
+    }
+
+    char buf[strlen (name) + 1];
+    memcpy (buf, name, sizeof buf);
+
+    if (! strncmp (buf, "file:///", 8))
+    {
+        char * base, * first, * second;
+        split_filename (skip_top_folders (buf), & base, & first, & second);
 
         if (! title)
-            title = base;
+            title = str_get_decoded (base);
 
-        for (gint i = 0; i < G_N_ELEMENTS (skip); i ++)
+        for (int i = 0; i < G_N_ELEMENTS (skip); i ++)
         {
-            if (first && ! strcasecmp (first, skip[i]))
+            if (first && ! g_ascii_strcasecmp (first, skip[i]))
                 first = NULL;
-            if (second && ! strcasecmp (second, skip[i]))
+            if (second && ! g_ascii_strcasecmp (second, skip[i]))
                 second = NULL;
         }
 
@@ -321,29 +374,29 @@ void describe_song (const gchar * name, const Tuple * tuple, gchar * * _title,
         {
             if (second && ! artist && ! album)
             {
-                artist = second;
-                album = first;
+                artist = str_get_decoded (second);
+                album = str_get_decoded (first);
             }
             else if (! artist)
-                artist = first;
+                artist = str_get_decoded (first);
             else if (! album)
-                album = first;
+                album = str_get_decoded (first);
         }
     }
     else
     {
         if (! title)
-            title = stream_name (copy);
+        {
+            title = str_get_decoded (stream_name (buf));
+
+            if (! title)
+                title = str_get_decoded (buf);
+        }
         else if (! artist)
-            artist = stream_name (copy);
+            artist = str_get_decoded (stream_name (buf));
         else if (! album)
-            album = stream_name (copy);
+            album = str_get_decoded (stream_name (buf));
     }
 
-DONE:
-    * _title = title ? stringpool_get ((gchar *) title, FALSE) : NULL;
-    * _artist = artist ? stringpool_get ((gchar *) artist, FALSE) : NULL;
-    * _album = album ? stringpool_get ((gchar *) album, FALSE) : NULL;
-
-    g_free (copy);
+    goto DONE;
 }
