@@ -1,22 +1,20 @@
 /*
- * libaudgui/util.c
- * Copyright 2010 John Lindgren
+ * util.c
+ * Copyright 2010-2011 John Lindgren
  *
- * This file is part of Audacious.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * Audacious is free software: you can redistribute it and/or modify it under
- * the terms of the GNU General Public License as published by the Free Software
- * Foundation, version 2 or version 3 of the License.
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions, and the following disclaimer.
  *
- * Audacious is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
- * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions, and the following disclaimer in the documentation
+ *    provided with the distribution.
  *
- * You should have received a copy of the GNU General Public License along with
- * Audacious. If not, see <http://www.gnu.org/licenses/>.
- *
- * The Audacious team does not consider modular code linking to Audacious or
- * using our public API to be a derived work.
+ * This software is provided "as is" and without any warranty, express or
+ * implied. In no event shall the authors be liable for any damages arising from
+ * the use of this software.
  */
 
 #include <gdk/gdkkeysyms.h>
@@ -27,22 +25,24 @@
 #include <audacious/gtk-compat.h>
 #include <audacious/i18n.h>
 #include <audacious/playlist.h>
-#include <audacious/plugin.h>
 #include <audacious/misc.h>
 #include <libaudcore/audstrings.h>
 #include <libaudcore/hook.h>
 
 #include "config.h"
+#include "init.h"
 #include "libaudgui.h"
 #include "libaudgui-gtk.h"
 
+static GdkPixbuf * current_pixbuf;
+
 void audgui_hide_on_delete (GtkWidget * widget)
 {
     g_signal_connect (widget, "delete-event", (GCallback)
      gtk_widget_hide_on_delete, NULL);
 }
 
-static gboolean escape_cb (GtkWidget * widget, GdkEventKey * event, void
+static bool_t escape_cb (GtkWidget * widget, GdkEventKey * event, void
  (* action) (GtkWidget * widget))
 {
     if (event->keyval == GDK_Escape)
@@ -57,38 +57,38 @@ static gboolean escape_cb (GtkWidget * widget, GdkEventKey * event, void
 void audgui_hide_on_escape (GtkWidget * widget)
 {
     g_signal_connect (widget, "key-press-event", (GCallback) escape_cb,
-     gtk_widget_hide);
+     (void *) gtk_widget_hide);
 }
 
 void audgui_destroy_on_escape (GtkWidget * widget)
 {
     g_signal_connect (widget, "key-press-event", (GCallback) escape_cb,
-     gtk_widget_destroy);
+     (void *) gtk_widget_destroy);
 }
 
-static void toggle_cb (GtkToggleButton * toggle, gboolean * setting)
+static void toggle_cb (GtkToggleButton * toggle, bool_t * setting)
 {
     * setting = gtk_toggle_button_get_active (toggle);
 }
 
-void audgui_connect_check_box (GtkWidget * box, gboolean * setting)
+void audgui_connect_check_box (GtkWidget * box, bool_t * setting)
 {
     gtk_toggle_button_set_active ((GtkToggleButton *) box, * setting);
     g_signal_connect ((GObject *) box, "toggled", (GCallback) toggle_cb, setting);
 }
 
 void audgui_simple_message (GtkWidget * * widget, GtkMessageType type,
- const gchar * title, const gchar * text)
+ const char * title, const char * text)
 {
     AUDDBG ("%s\n", text);
 
     if (* widget != NULL)
     {
-        const gchar * old = NULL;
+        const char * old = NULL;
         g_object_get ((GObject *) * widget, "text", & old, NULL);
         g_return_if_fail (old);
 
-        gint messages = GPOINTER_TO_INT (g_object_get_data ((GObject *)
+        int messages = GPOINTER_TO_INT (g_object_get_data ((GObject *)
          * widget, "messages"));
 
         if (messages > 10)
@@ -97,7 +97,7 @@ void audgui_simple_message (GtkWidget * * widget, GtkMessageType type,
         if (strstr (old, text))
             goto CREATED;
 
-        gchar both[strlen (old) + strlen (text) + 2];
+        char both[strlen (old) + strlen (text) + 2];
         snprintf (both, sizeof both, "%s\n%s", old, text);
         g_object_set ((GObject *) * widget, "text", both, NULL);
 
@@ -121,7 +121,7 @@ CREATED:
     gtk_window_present ((GtkWindow *) * widget);
 }
 
-GdkPixbuf * audgui_pixbuf_from_data (void * data, gint size)
+GdkPixbuf * audgui_pixbuf_from_data (const void * data, int64_t size)
 {
     GdkPixbuf * pixbuf = NULL;
     GdkPixbufLoader * loader = gdk_pixbuf_loader_new ();
@@ -140,56 +140,35 @@ GdkPixbuf * audgui_pixbuf_from_data (void * data, gint size)
     return pixbuf;
 }
 
-GdkPixbuf * audgui_pixbuf_for_entry (gint list, gint entry)
+GdkPixbuf * audgui_pixbuf_for_entry (int list, int entry)
 {
-    gchar * name = aud_playlist_entry_get_filename (list, entry);
+    char * name = aud_playlist_entry_get_filename (list, entry);
     g_return_val_if_fail (name, NULL);
 
-    /* Don't get album art for network files -- too slow. */
-    if (! strncmp (name, "http://", 7) || ! strncmp (name, "https://", 8) ||
-     ! strncmp (name, "mms://", 6))
-        goto FALLBACK;
+    const void * data;
+    int64_t size;
 
-    AUDDBG ("Trying to load pixbuf for %s.\n", name);
-    PluginHandle * decoder = aud_playlist_entry_get_decoder (list, entry, FALSE);
-    if (! decoder)
-        goto FALLBACK;
+    aud_art_get_data (name, & data, & size);
 
-    void * data;
-    gint size;
-
-    if (aud_file_read_image (name, decoder, & data, & size))
+    if (data)
     {
         GdkPixbuf * p = audgui_pixbuf_from_data (data, size);
-        g_free (data);
+        aud_art_unref (name);
+
         if (p)
         {
-            g_free (name);
+            str_unref (name);
             return p;
         }
     }
 
-    gchar * assoc = aud_get_associated_image_file (name);
+    str_unref (name);
 
-    if (assoc)
-    {
-        GdkPixbuf * p = gdk_pixbuf_new_from_file (assoc, NULL);
-        g_free (assoc);
-        if (p)
-        {
-            g_free (name);
-            return p;
-        }
-    }
-
-    g_free (name);
-
-FALLBACK:;
     AUDDBG ("Using fallback pixbuf.\n");
     static GdkPixbuf * fallback = NULL;
     if (! fallback)
     {
-        gchar * path = g_strdup_printf ("%s/images/album.png",
+        char * path = g_strdup_printf ("%s/images/album.png",
          aud_get_path (AUD_PATH_DATA_DIR));
         fallback = gdk_pixbuf_new_from_file (path, NULL);
         g_free (path);
@@ -199,49 +178,33 @@ FALLBACK:;
     return fallback;
 }
 
-static void pixbuf_clear_cb (void * unused, GdkPixbuf * * pixbuf)
+void audgui_pixbuf_uncache (void)
 {
-    if (* pixbuf)
+    if (current_pixbuf)
     {
-        AUDDBG ("Clearing cached pixbuf.\n");
-        g_object_unref ((GObject *) * pixbuf);
-        * pixbuf = NULL;
+        g_object_unref ((GObject *) current_pixbuf);
+        current_pixbuf = NULL;
     }
 }
 
-static void pixbuf_position_cb (void * list, GdkPixbuf * * pixbuf)
-{
-    if (GPOINTER_TO_INT (list) == aud_playlist_get_playing ())
-        pixbuf_clear_cb (NULL, pixbuf);
-}
-
 GdkPixbuf * audgui_pixbuf_for_current (void)
 {
-    static GdkPixbuf * pixbuf = NULL;
-    static gboolean hooked = FALSE;
-
-    if (! hooked)
+    if (! current_pixbuf)
     {
-        hook_associate ("playlist set playing", (HookFunction) pixbuf_clear_cb, & pixbuf);
-        hook_associate ("playlist position", (HookFunction) pixbuf_position_cb, & pixbuf);
-        hooked = TRUE;
+        int list = aud_playlist_get_playing ();
+        current_pixbuf = audgui_pixbuf_for_entry (list, aud_playlist_get_position (list));
     }
 
-    if (! pixbuf)
-    {
-        gint list = aud_playlist_get_playing ();
-        pixbuf = audgui_pixbuf_for_entry (list, aud_playlist_get_position (list));
-    }
+    if (current_pixbuf)
+        g_object_ref ((GObject *) current_pixbuf);
 
-    if (pixbuf)
-        g_object_ref ((GObject *) pixbuf);
-    return pixbuf;
+    return current_pixbuf;
 }
 
-void audgui_pixbuf_scale_within (GdkPixbuf * * pixbuf, gint size)
+void audgui_pixbuf_scale_within (GdkPixbuf * * pixbuf, int size)
 {
-    gint width = gdk_pixbuf_get_width (* pixbuf);
-    gint height = gdk_pixbuf_get_height (* pixbuf);
+    int width = gdk_pixbuf_get_width (* pixbuf);
+    int height = gdk_pixbuf_get_height (* pixbuf);
     GdkPixbuf * pixbuf2;
 
     if (width > height)
@@ -265,3 +228,10 @@ void audgui_pixbuf_scale_within (GdkPixbuf * * pixbuf, gint size)
     g_object_unref (* pixbuf);
     * pixbuf = pixbuf2;
 }
+
+void audgui_set_default_icon (void)
+{
+#ifndef _WIN32
+    gtk_window_set_default_icon_name ("audacious");
+#endif
+}
